import * as d3 from "d3";
import React, {useEffect, useState} from "react";
import {DonutChartLegend} from "../../components/d3js/Axis";

const items = [
    {region: "Bretagne", value: 0.05},
    {region: "Nouvelle Aquitaine", value: 0.22},
    {region: "Occitanie", value: 0.16},
    {region: "Centre Val de Loire", value: 0.08},
    {region: "Normandie", value: 0.115},
    {region: "Ile de France", value: 0.13},
    {region: "Grand Est", value: 0.12},
    {region: "Autre", value: 0.125}
]
export default function Regions(){
    const radius = Math.min(1000, 1000) / 2 - 50
    const [arcs, setArcs] = useState<{name: string, value: number, angle: {startAngle: number, endAngle: number}, color: string}[]>([])

    //Génère une couleur aléatoire
    const randomColor = () => {
        var letters = '0123456789ABCDEF';
        var color = '#';
        for (var i = 0; i < 6; i++) {
            color += letters[Math.floor(Math.random() * 16)];
        }
        return color;
    }

    useEffect(() => {
        let start=0;
        setArcs(items.map(item => {
            let startAngle = start;
            let endAngle = startAngle + item.value*(Math.PI*2);
            start = endAngle;
            console.log(`${item.region} : ${startAngle}:${endAngle}`);
            return {
                name: item.region,
                value: item.value,
                angle:{
                    startAngle: startAngle,
                    endAngle: endAngle
                },
                color: randomColor()
            }
        }));
    }, [])

    return (
        <div className="d-flex justify-content-center">
            <svg
                width="0"
                height="0"
                overflow="visible"
            >
                <DonutChart data={arcs} radius={radius} />
                <DonutChartLegend data={arcs} radius={radius} />
            </svg>
        </div>
    )
}

function DonutChart({data, radius}:{data: {name: string, value: number, angle: {startAngle: number, endAngle: number}, color: string}[], radius: number}){
    //Génère un arc par rapport à un startAngle et endAngle
    const arc = d3.arc().innerRadius(radius * 0.5).outerRadius(radius * 0.8);

    //Détermine la couleur d'un texte par rapport à la couleur du background
    const getContrast = (hexcolor: string) => {
        var r = parseInt(hexcolor.substring(1,3),16);
        var g = parseInt(hexcolor.substring(3,5),16);
        var b = parseInt(hexcolor.substring(5,7),16);
        var yiq = ((r*299)+(g*587)+(b*114))/1000;
        return (yiq >= 128) ? 'black' : 'white';
    }
    return (
        <g>
            {data.map((item) => (
                <g
                    key={item.name}
                >
                    {/*Arc représentant la valeur de l'item*/}
                    <path
                        d={arc(item.angle)}
                        fill={item.color}
                    />
                    {/*Texte indiquant la valeur de l'item*/}
                    <g
                        transform={`translate(${arc.centroid(item.angle)})`}
                    >
                        <text
                            textAnchor="middle"
                            fill={getContrast(item.color)}
                        >{item.value*100} %</text>
                    </g>
                </g>
            ))}
        </g>
    )
}